#ifndef _DT_H_
#define _DT_H_

#define TRUE 1
#define FALSE 0

/**
 * Une énumération de types
 */
typedef enum {
    TYPE_INVALID,
    TYPE_STRING,
    TYPE_INT,
    TYPE_BOOLEAN
} Type;

typedef struct _DataTable DataTable;
typedef struct _DataContent DataContent;
typedef struct _DataCase DataCase;

typedef void* pointer;

/**
 * Notre tableau
 */
struct _DataTable {
    size_t length;
    DataCase *head;
    DataCase *tail;
};

/**
 * Le contenu d'une case
 */
struct _DataContent {
    union {
        char *string;
        int iNumber;
        short booleen;
    };
    Type type;
};

/**
 * Une case du tableau
 */
struct _DataCase {
    DataContent *content;
    DataCase *next;
    DataCase *prev;
};

#define datacontent_new_string(val) datacontent_new(TYPE_STRING, (void*)val)
#define datacontent_new_int(val) datacontent_new(TYPE_INT, (void*)val)
#define datacontent_new_boolean(val) datacontent_new(TYPE_BOOLEAN, (void*)val)

#define free_datacase(val) free_datacase_full(val, NULL)

#define free_datacontent(val) free_datacontent_full(val, NULL)

#define INT_TO_POINTER(i) ((pointer) (long) (i))
#define POINTER_TO_INT(p) ((int)  (long) (p))


typedef void (* DataAction) (DataCase *pCase, pointer *pData);

void free_datatable (DataTable **p_list);
void datacase_print (DataCase *pCase, pointer *pData);
void datatable_foreach (DataTable *p_list, DataAction func, pointer *pData);
void free_datacontent_full (DataContent *pContent, pointer *pData);
void free_datacase_full (DataCase *pCase, pointer *pData);
short datacontent_equals (DataContent *d1, DataContent *d2);
void datatable_remove_id(DataTable **p_list, int position);
void datatable_remove_all(DataTable **p_list, DataContent *data);
void datatable_remove(DataTable **p_list, DataContent *data);
size_t datatable_length(DataTable *p_list);
void datatable_insert(DataTable **p_list, DataContent *data, int position);
void datatable_prepend(DataTable **p_list, DataContent *data);
void datatable_append(DataTable **p_list, DataContent *data);
DataTable *create_datatable (Type iDataType, ...);
DataContent *datacontent_new (Type iType, void *value);

#endif
